# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# groupvte 
# Copyright (C) 2005 Michele Baldessari <michele@pupazzo.org>

# This file may be distributed and/or modified under the terms of
# the GNU General Public License version 2 as published by
# the Free Software Foundation.
# This file is distributed without any warranty; without even the implied
# warranty of merchantability or fitness for a particular purpose.
# See "LICENSE.GPL" in the source distribution for more information.

import gtk
import gtk.gdk
import gconf
import sys
import os.path
import StringIO
import traceback
import pango
import logging
from vteview import *
from settings import Settings, SettingsUI
from types import DictType, ListType
        
def trace_exception(type, value, t):
    trace = StringIO.StringIO()
    traceback.print_exception(type, value, t, None, trace)
    return trace.getvalue()

class ExceptionDialog:
    def __init__(self, parent, traceback):
        dialog = gtk.Dialog("Unknown error", parent, 
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            ( gtk.STOCK_QUIT, gtk.RESPONSE_CANCEL , 
              gtk.STOCK_OK, gtk.RESPONSE_OK))
        dialog.set_resizable(False)
        dialog.set_border_width(12)
        dialog.vbox.set_spacing(6)
        image = gtk.Image()
        image.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
        dialog.vbox.pack_start(image)

        description = gtk.Label( "An unknown error occured. Please report the text below, " \
            "along with what you were doing that may have caused the error.")
        description.set_justify(gtk.JUSTIFY_FILL)
        description.set_line_wrap(True)
        dialog.vbox.pack_start(description)

        textview = gtk.TextView()
        textview.get_buffer().set_text(traceback)
        textview.modify_font(pango.FontDescription("Monospace"))
        scrolledwindow = gtk.ScrolledWindow()
        scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolledwindow.add(textview)
                                                                    
        scrolledwindow.set_size_request(-1, 120)
        dialog.vbox.pack_start(scrolledwindow)
        self.dialog = dialog


    def run(self):
        self.dialog.show_all()
        return self.dialog.run() == gtk.RESPONSE_OK


class MultiTerminal:
    ui = '''
    <ui>
      <menubar name="MenuBar">
        <menu action="Main">
          <menuitem action="Switch session"/>
          <menuitem action="Quit"/>
        </menu>
        <menu action="Edit">
          <menuitem action="Copy"/>
          <menuitem action="Paste"/>
          <separator/>
          <menuitem action="Preferences"/>
        </menu>
        <menu action="Tabs">
          <menuitem action="New"/>
          <menuitem action="Reset"/>
        </menu>
        <menu action="View">
          <menuitem action="Notebook"/>
          <menuitem action="Table"/>
        </menu>
        <placeholder name="SessionsHere">
        </placeholder>
        <menu action="Help">
          <menuitem action="About"/>
        </menu>
      </menubar>
    </ui>'''

	def modified_cb(self, terminal, l):
		if (self.vteview.get_current_tab() != l):
            self.vteview.set_label_color(l, "red")
		else:
            self.vteview.set_label_color(l, "black")
		
	def switch_page_cb(self, page, page_num, tab_number):
        self.vteview.set_label_color(tab_number, "black")

	def child_exited_cb(self, terminal, t):
        logging.debug("to remove terminal %s t %s" % (terminal, t))
        #if (len(self.terminals) > 1):
        #    self.vteview.remove_tab(t)
        #    del self.terminals[t]

    def copy_clipboard_cb(self):
        logging.debug("copy")
    
    def paste_clipboard_cb(self):
        logging.debug("paste")

	def add_terminal(self, label, commands=None):
		# Create VTE widget and set some defaults
        vt = VteStruct()
        
		vt.term.connect("child-exited", self.child_exited_cb, len(self.terminals))
		vt.term.connect("cursor-moved", self.modified_cb, len(self.terminals))
		vt.term.connect("key_press_event", self.on_terminal_key_event)
 
		self.terminals.append(vt)
		vt.scrollbar.set_adjustment(self.terminals[-1].term.get_adjustment())
		vt.hbox.pack_start(self.terminals[-1].term, True, True)
		vt.hbox.pack_start(self.terminals[-1].scrollbar, False, False)
        vt.label.set_text(label + " - " + str(len(self.terminals)))
		self.vteview.append_tab(vt, self.window.get_size())
        self.vteview.set_current_tab(0)
		self.window.show_all()
		if commands != None:
			for i in commands:
                if not i.endswith("\n"):
                    vt.term.feed_child(i + "\n")
                else:
                    vt.term.feed_child(i)
	
	def execute(self):
		s = self.entry.get_text()
		if s.strip() != "":
			for i in self.terminals:
				i.term.feed_child(self.entry.get_text() + "\n")
			self.entry.set_text("")

	def on_key_press_event_enter_cb(self, widget, event):
		keyname = gtk.gdk.keyval_name(event.keyval)
		if (keyname == "Return"):
			self.execute()
		
	def button_clicked_cb(self, widget, data):
		self.execute()

	def new_terminal_tab_cb(self, widget):
		self.add_terminal("New")

    def on_terminal_key_event(self, widget, event):
        mods = []
        keyname = gtk.gdk.keyval_name(event.keyval)
        if event.state & gtk.gdk.CONTROL_MASK:
            mods.append('ctrl')
        if event.state & gtk.gdk.SHIFT_MASK:
            mods.append('shift')
        
        mod = '_'.join(mods)
        if mod:
            name = '%s_%s' % (mod, keyname)
        else:
            name = '%s' % keyname

        if name == "ctrl_shift_C":
            if widget.get_has_selection():
                widget.copy_clipboard()
            return True
        elif name == "ctrl_shift_V":
            widget.paste_clipboard()
            return True
        return False

    def copy(self, widget):
        n = self.vteview.get_current_tab()
        vt = self.terminals[n].term
        if vt.get_has_selection():
            vt.copy_clipboard()
        return

    def paste(self, widget):
        n = self.vteview.get_current_tab()
        vt = self.terminals[n].term
        vt.paste_clipboard()
        return

    def on_key_press_event(self, widget, event):
        keyname = gtk.gdk.keyval_name(event.keyval)
        if event.state & gtk.gdk.MOD1_MASK:
            if keyname in ['1','2','3','4','5','6','7','8','9']:
                key = int(keyname)
                if key <= len(self.terminals):
                    self.vteview.set_current_tab(key - 1)
                    self.terminals[key - 1].term.grab_focus()
                    return True
        return False

    def about(self, action):
        dialog = gtk.AboutDialog()
        dialog.set_version(self.info['version'])
        dialog.set_copyright("Michele Baldessari (c) 2005")
        my_license = """
        This file may be distributed and/or modified under the terms of
        the GNU General Public License version 2 as published by
        the Free Software Foundation.
        This file is distributed without any warranty; without even the implied
        warranty of merchantability or fitness for a particular purpose.
        See "LICENSE.GPL" in the source distribution for more information.

        """
        dialog.set_license(my_license)
        dialog.set_website("http://michele.pupazzo.org/groupvte")
        dialog.set_authors(["Michele Baldessari <michele@pupazzo.org>"])
        dialog.set_artists(["Jakub Steiner"])
        image = gtk.gdk.pixbuf_new_from_file(os.path.join(self.info['pixmapdir'], 'groupvte.png'))
        dialog.set_logo(image)

        dialog.run()
        dialog.destroy()
    
    def delete_event(self, widget, event, data=None):
        self.quit(event)
        return False

    def quit(self, action):
        if len(self.terminals) > 0:
            dialog = gtk.Dialog('Quit', self.window,
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_QUIT, gtk.RESPONSE_ACCEPT))
            dialog.set_resizable(False)
            dialog.set_border_width(12)
            dialog.vbox.set_spacing(6)
            image = gtk.Image()
            image.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
            description = gtk.Label('There are <b>%d</b> terminals open\n' \
                'Are you sure you want to quit?' % (len(self.terminals)))
            description.set_use_markup(True)
            description.set_justify(gtk.JUSTIFY_FILL)
            description.set_line_wrap(True)

            dialog.vbox.pack_start(image)
            dialog.vbox.pack_start(description)

            dialog.vbox.show_all()
            r = dialog.run()
            dialog.destroy()
            if r == gtk.RESPONSE_ACCEPT:
                gtk.main_quit()
        else:
            gtk.main_quit()

    def change_view_cb(self, action, current):
        choice = action.get_current_value()
        if choice == self.vteview.currentview:
            return

        self.vteview.remove_all_tabs()
        self.vteview.destroy_view()

        if choice == VteView.VIEW_NOTEBOOK:
            self.vteview = VteView(VteView.VIEW_NOTEBOOK, self.terminals)
            self.currentview = VteView.VIEW_NOTEBOOK
        elif choice ==  VteView.VIEW_TABLE:
            self.vteview = VteView(VteView.VIEW_TABLE, self.terminals)
            self.currentview = VteView.VIEW_TABLE
 
		self.vbox.pack_start(self.vteview.get_view(), True, True, 0)
        self.vbox.reorder_child(self.vteview.get_view(), 1)
 
    def reset_tabs_cb(self, w):
        for i in self.terminals:
            i.term.reset(True, True)
            i.term.feed_child("\n")

    def preferences_cb(self, w):
        SettingsUI(self.window, self.info, self.conf)

    def new_session(self, session):
        if not self.sessions.has_key(session):
            return

        items = self.sessions[session]
        logging.debug("items %s" % items)
        for v in items:
            logging.debug("v %s" % v)
            connect = self.machines[v]
            logging.debug("connect %s" % connect)
            self.add_terminal(v, connect)

    def open_new_session(self, newsessionname):
        if not self.sessions.has_key(newsessionname):
            return
        logging.debug("Opening session %s" % newsessionname)
        self.vteview = VteView(self.currentview)
        self.new_session(newsessionname)

        self.vteview = VteView(self.currentview, self.terminals)
		self.vbox.pack_start(self.vteview.get_view(), True, True, 0)
        self.vbox.reorder_child(self.vteview.get_view(), 1)
        self.cur_session = newsessionname

    def close_current_session(self):
        logging.debug("close_current_session %s" % self.cur_session)
        self.vteview.destroy_view()
        
        for i in self.terminals:
            logging.debug("killing %s" % i.term)
            del i.term
            logging.debug("killing %s" % i.hbox)
            del i.hbox
            logging.debug("killing %s" % i.label)
            del i.label
            del i

        self.terminals = []
        del self.vteview

    def session_cb(self, widget):
        dialog = gtk.Dialog('Switch session', self.window,
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        dialog.set_resizable(False)
        dialog.set_border_width(12)
        dialog.vbox.set_spacing(6)
        image = gtk.Image()
        image.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
        label = gtk.Label("Choose session:")

        combobox = gtk.combo_box_new_text()
        counter  = 0
        active   = 0
        for i in self.conf.get_session_list():
            combobox.append_text(i)
            if i == self.cur_session:
                active = counter
            counter = counter + 1

        combobox.set_active(active)
        dialog.vbox.pack_start(image)
        dialog.vbox.pack_start(label)
        dialog.vbox.pack_start(combobox)

        dialog.vbox.show_all()
        r = dialog.run()
        dialog.destroy()
        if r == gtk.RESPONSE_REJECT:
            return

        model = combobox.get_model()
        index = combobox.get_active()
        newsession = model[index][0]
        if newsession == self.cur_session:
            return

        logging.debug("Close current session")
        self.close_current_session()

        logging.debug("Open new session %s" % newsession)
        self.open_new_session(newsession)

    def init_vbox(self):
		self.entry = gtk.Entry()
        self.entry.connect('key_press_event', self.on_key_press_event_enter_cb)
        map = self.entry.get_colormap()
        colour = map.alloc_color("#ed9191")

        style = self.entry.get_style().copy()
        style.base[gtk.STATE_NORMAL] = colour
        self.entry.set_style(style)
 
        self.ui_manager = gtk.UIManager()
		accel_group = self.ui_manager.get_accel_group()
        self.window.add_accel_group(accel_group)
        actiongroup = gtk.ActionGroup('Groupvte')

        # Create actions
        actiongroup.add_actions(
            [('Quit',     gtk.STOCK_QUIT,     '_Quit',     
                None, 'Quit the Program',     self.quit),
             ('Switch session', None          ,'_Switch session',
                None, 'Switch session',  self.session_cb),
             ('Main',     None,               '_Main'),
             ('Edit',     None,               '_Edit'),
             ('Copy',     gtk.STOCK_COPY,     '_Copy',     
                None, 'Copy from clipboard',  self.copy),
             ('Paste',    gtk.STOCK_PASTE,    '_Paste',    
                None, 'Paste from clipboard', self.paste),
             ('Preferences', gtk.STOCK_PREFERENCES, '_Preferences', 
                None, 'Edit preferences..',self.preferences_cb),
             ('Tabs',     None,               '_Tabs'),
             ('New',      gtk.STOCK_NEW,      '_New',
                 None, 'New terminal tab..',   self.new_terminal_tab_cb),
             ('Reset',    gtk.STOCK_REFRESH,  '_Reset',    
                None, 'Reset all tabs..',     self.reset_tabs_cb),
             ('View',     None,               '_View'),
             ('Sessions', None,               '_Sessions'),
             ('Empty',    None,               '_Empty'),
             ('Help',     None,               '_Help'),
             ('About',    None,               '_About',    
                 None, 'About Gropuvte..',     self.about)])

        actiongroup.add_radio_actions(
            [('Notebook', None, '_Notebook', None, 'Notebook View', VteView.VIEW_NOTEBOOK),
             ('Table',    None, '_Table',    None, 'Table View',    VteView.VIEW_TABLE), ], 
             self.currentview, self.change_view_cb)
        

        actiongroup.get_action('Quit').set_property('short-label', '_Quit')
		self.vbox = gtk.VBox()
        self.ui_manager.insert_action_group(actiongroup, 0)
        self.ui_manager.add_ui_from_string(self.ui)
        self.menubar = self.ui_manager.get_widget('/MenuBar')
        helpmenu = self.ui_manager.get_widget('/MenuBar/Help')
        helpmenu.set_right_justified(True)

        self.vbox.pack_start(self.menubar, False, True, 0)
		self.vbox.pack_start(self.vteview.get_view(), True, True, 0)
		self.hbox = gtk.HBox()
		b = gtk.Button("", gtk.STOCK_EXECUTE)
		b.connect("clicked", self.button_clicked_cb, "")
		self.hbox.pack_start(self.entry, True, True)
		self.hbox.pack_start(b, False, False, 0)

		self.vbox.pack_start(self.hbox, False, False)
		self.window.add(self.vbox)
		self.window.connect("delete_event", self.delete_event)
		self.entry.grab_focus()
        self.window.connect("key_press_event", self.on_key_press_event)
		self.window.show_all()
            
    def __cb_exception(self, type, value, trace):
        if type == KeyboardInterrupt:
            return

        traceback = trace_exception(type, value, trace)
        sys.stderr.write(traceback)

        if ExceptionDialog(self.window, traceback).run():
            self.window.run()
        else:
            sys.exit(1)

	def __init__(self, info, conf):
        sys.excepthook = self.__cb_exception
        
		self.info        = info
		self.window      = gtk.Window()
        self.conf        = conf
		self.terminals   = []
        self.sessions    = conf.get_session_hash()
        self.machines    = conf.get_machine_hash()
        if info["session"] == "":
            self.cur_session = conf.get_default_session()
        else:
            if not self.conf.has_session(info["session"]):
                logging.error("Session %s does not exist" % (info["session"]))
                sys.exit(-1)

            self.cur_session = info["session"]

        logging.info("Sessions    : %s" % self.sessions)
        logging.info("Machines    : %s" % self.machines)

        if self.conf.get_view() == Settings._notebookview:
    		self.vteview   = VteView(VteView.VIEW_NOTEBOOK)
            self.currentview = VteView.VIEW_NOTEBOOK
        elif self.conf.get_view() == Settings._tableview:
    		self.vteview   = VteView(VteView.VIEW_TABLE)
            self.currentview = VteView.VIEW_TABLE
        else:
            logging.error("Error view must be \"notebook\" or \"table\"")
            sys.exit(2)

        self.init_vbox()
        self.new_session(self.cur_session)

